@@ -17,6 +17,10 @@ function pad2(x) {
     return x.toString().padStart(2, "0");
 }
 
+function zip(a, b) {
+    return Array.from(Array(Math.max(b.length, a.length)), (_, i) => [a[i], b[i]]);
+}
+
 function dtf(dt, format) {
     return format
         .replace("YYYY", dt.getFullYear())
@@ -255,16 +259,15 @@ function markdownToHtmlFancy(element, useKatex=false) {
     element.querySelectorAll("p").forEach(paragraph => {
         paragraph.innerHTML = paragraph.innerHTML.replace(/(\w) ([:\?!;»€°])/g, "$1 $2").replace(/([«°]) (\w)/g, "$1 $2");;
     });
-    element.querySelectorAll(".reference").forEach(noteReference => {
-        fetch(URL_API + `?action=note-title&objectId=${noteReference.getAttribute("ref-id")}`).then(res => {
-            if (res.status == 404) {
-                return "<404 Not Found>";
-            } else {
-                return res.text();
-            }
-        }).then(text => {
-            noteReference.textContent = text;
-        });
+    const noteReferences = element.querySelectorAll(".reference");
+    const noteReferenceIds = [];
+    for (const noteReference of noteReferences) {
+        noteReferenceIds.push(noteReference.getAttribute("ref-id"));
+    }
+    fetch(URL_API + `?action=note-title&objectIds=${noteReferenceIds.join(",")}`).then(res => res.json()).then(data => {
+        for (const [noteReference, noteTitle] of zip(noteReferences, data.titles)) {
+            noteReference.textContent = noteTitle == null ? "<404 Not Found>" : noteTitle;
+        }
     });
     window.addEventListener("load", () => {
         hljs.highlightAll();
@@ -384,7 +387,7 @@ function bindDropdown(dropdown) {
         const menuBounds = menu.getBoundingClientRect();
         const fitsUnder = toggleBounds.bottom + padding + menuBounds.height <= window.innerHeight;
         const baseLeft = toggleBounds.left;
-        const maxLeft = window.innerWidth - padding - menuBounds.width;
+        const maxLeft = screen.width - padding - menuBounds.width;
         const left = Math.max(padding, Math.min(baseLeft, maxLeft));
         menu.style.left = left + "px";
         if (fitsUnder) {
@@ -410,9 +413,7 @@ function bindGotoPaginatorButton(buttonSelector, currentPage, maxPage, attrStrin
         const page = prompt(`Go to page (1-${maxPage}):`, currentPage.toString());
         if (page) {
             const pageNumber = Math.max(1, Math.min(maxPage, parseInt(page)));
-            console.log(pageNumber);
             const baseUrl = window.location.protocol + '//' + window.location.host + window.location.pathname;
-            console.log(baseUrl);
             const newUrl = baseUrl + `?page=${pageNumber}${attrString}`;
             window.location.href = newUrl;
         }
@@ -564,6 +565,88 @@ function setupCategoryInput() {
     updateCategories();
 }
 
+function urlBase64ToUint8Array(base64String) {
+    const padding = '='.repeat((4 - base64String.length % 4) % 4);
+    const base64 = (base64String + padding)
+    .replace(/-/g, '+')
+    .replace(/_/g, '/');
+
+    const rawData = atob(base64);
+    const outputArray = new Uint8Array(rawData.length);
+
+    for (let i = 0; i < rawData.length; ++i) {
+    outputArray[i] = rawData.charCodeAt(i);
+    }
+    return outputArray;
+}
+
+function bindEditable(editable) {
+    const objectId = editable.getAttribute("object_id");
+    const apiAction = editable.getAttribute("action");
+    var editableText;
+    function rebuildEditable(value) {
+        editable.innerHTML = "";
+        editableText = create(editable, "span", "editable-text");
+        editableText.textContent = value;
+        editableText.addEventListener("click", onEditableTextClick);
+    }
+    function onEditableTextClick() {
+        const currentValue = editable.textContent;
+        const input = document.createElement("input");
+        input.className = "editable-input input-slim";
+        input.size = Math.floor(currentValue.length * 1.1);
+        input.value = currentValue;
+        if (editableText == undefined) throw new Error();
+        editableText.replaceWith(input);
+        input.focus();
+        function submitInput() {
+            let newValue = input.value.trim();
+            if (newValue == "") {
+                rebuildEditable(currentValue);
+            } else {
+                apiPost(apiAction, {id: objectId, value: newValue}, () => {
+                    rebuildEditable(newValue);
+                    toast("Renamed subscription", TOAST_SHORT);
+                });
+            }
+        }
+        input.addEventListener("focusout", submitInput);
+        input.addEventListener("keydown", (event) => {
+            if (event.key == "Enter") {
+                submitInput();
+            }
+        });
+    }
+    const defaultValue = editable.textContent;
+    rebuildEditable(defaultValue);
+}
+
+function askUserToSubscribeToNotifications(serviceWorkerUrl, publicKeyBase64) {
+    if ("serviceWorker" in navigator && "PushManager" in window) {
+        navigator.serviceWorker.register(serviceWorkerUrl).then(function(registration) {
+            console.log("Service Worker registered with scope:", registration.scope);
+            Notification.requestPermission().then(async function(permission) {
+                if (permission === "granted") {
+                    console.log("User accepted notifications");
+                    registration.pushManager.subscribe({
+                        userVisibleOnly: true,
+                        applicationServerKey: urlBase64ToUint8Array(publicKeyBase64)
+                    }).then(function(subscription) {
+                        const body = JSON.stringify(subscription);
+                        apiPost("save-subscription", {"subscription": body}, () => {
+                            window.location.reload();
+                        });
+                    });
+                } else {
+                    console.log("User rejected notifications");
+                }
+            });
+        });
+    } else {
+        console.error("Could not ask user for notifications, not supported");
+    }
+}
+
 window.addEventListener("load", () => {
     document.querySelectorAll(".link-confirm").forEach(link => {
         link.addEventListener("click", (event) => {
@@ -601,4 +684,5 @@ window.addEventListener("load", () => {
             }
         });
     });
+    document.querySelectorAll(".editable").forEach(bindEditable);
 });
\ No newline at end of file
